// Last modified: 6. August 1999

#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include "Item.h"
#include "Spell.h"

#define NAMESIZE 11  // size of name array
#define MAXSPELLS 10 // maximum number of spells memorized
#define SIZE 20

class Myr
{
public:

  // member functions

  // constructor
  Myr(int Ess = 50, int Lif = 30, int nmitem = 0);
  ~Myr(); // deconstructor

  void modEssence(int);
  void setEssence(int);
  int  retEssence(); // return essence points
  void modLife(int); // modify Life Points
  void setLife(int); // set Life Points for particular number
  int  retLife(); // return # of Life Points
  void setCircle(int);
  int  retCircle();

  // new inventory member functions
  void insertInv(char [], int); // add an item
  void removeInv(char []); // remove an item
  void printInv(); // print items in inventory
  int  searchInv(char []); // search the inventory for item

  void incnumitems();
  void decnumitems();
  int  searchitem(char *);
  void stats();
  void nope(char *); // cannot cast spell
  void incVar(int &); // increment variable
  void decVar(int &); // decrement variable
  void setVar(int &, int); // set integer variable

  // member functions for spells
  void addSpell(char *, int); // spell name & spell cost
  void castSpell(char *, int &);
  int  findSpell(char *);
  void setNumspells(int); // set numspells
  int  retNumspells(); // return number of spells known
  char * retSpellname(int); // return spell name
  int  retSpellcost(int); // return spell cost
  int  retSpellState(char *);

  // member functions for combat
  int retCS();
  int retWS();
  int retDefense();
  void setBonus(int);
  int retBonus();

private:
  // class variables
  char name[NAMESIZE];
  int  essence; // essence points
  int  maxessence; // maximum essence points
  int  life; // life points
  int  maxlife; // maximum life points
  int  circle;
  Item invent[15]; // linked list inventory
  List listinventory;
  int numitems; // number of items in inventory
  int diffitems; // number of different items
  Spell spell_array[MAXSPELLS];
  int numspells; // # of spells known
  int translate; // boolean value to tell if Translate spell is on

  // combat variables
  int cs; // combat skill
  int ws; // weapon skill
  int defense; // defense skill
  int bonus; // bonus

};

/*********************************************************
 * Myr(int Ess, int Lif, int nmitem)                     *
 * Myr constructor and initializer                       *
 ********************************************************/
Myr :: Myr(int Ess, int Lif, int nmitem)
{
  essence = Ess;
  maxessence = Ess;
  life = Lif;
  maxlife = Lif;
  numitems = nmitem;
  diffitems = 0;
  numspells = 0;
  translate = 0; // initially do not have Translate spell yet
  cs = 15;
  ws = 5;
  defense = 7;
  bonus = 0;
}

/*********************************************************
 * ~Myr()                                                *
 * Myr destructor                                        *
 ********************************************************/
Myr :: ~Myr()
{}


/*********************************************************
 * modEssence(int num)                                   *
 * num can be both negative or positive to modify EP     *
 ********************************************************/
// Note: the num can also be negative in both modEssence and modLife
void Myr :: modEssence(int num)
{
  essence = essence + num;
}

/*********************************************************
 * setEssence(int num)                                   *
 * set Essence for a particular number, useful for       *
 * restoring games                                       *
 ********************************************************/
// set Essence for a particular number, useful for restoring
void Myr :: setEssence(int num)
{
  essence = num;
}

/*********************************************************
 * retEssence()                                          *
 * return Essence points                                 *
 ********************************************************/
int Myr :: retEssence()
{
  return essence;
}

void Myr :: modLife(int num)
{
  life = life + num;

  // check to see if Myrlandria is dead
  if (life <= 0)
  {
    cout << "You are dead.  Thanks for playing, though." << endl;
    exit(1); // boot out of the program ungracefully
  }
  
  if (life > maxlife)
    life = maxlife; // make sure life points do not exceed maxlife
}

// set Life for a particular number, useful for restoring
void Myr :: setLife(int num)
{
  life = num;
}

int Myr :: retLife()
{
  return life;
}

void Myr :: setCircle(int num)
{
  circle = num;
}

int Myr :: retCircle()
{
  return circle;
}


// ----------------------------------------------------------------
// INVENTORY METHODS
// ----------------------------------------------------------------

// insert an item into the inventory
void Myr :: insertInv(char itemname[], int num)
{
  listinventory.insert(itemname, num);
  incVar(numitems);
}

void Myr :: removeInv(char token2[])
{
  listinventory.remove(token2);
  decnumitems();
}

void Myr :: printInv()
{
  listinventory.print();
}

int Myr :: searchInv(char token2[])
{
  return (listinventory.search(token2));
}

//-------------------------------------------------


void Myr :: incnumitems()
{
  numitems++;
}

void Myr :: decnumitems()
{
  numitems--;
}

int Myr :: searchitem(char * s)
{
  int key = -1;

  if (diffitems > 0)
  {
    for (int i = 0; (i < diffitems); i++)
    {
      if(!strcmp(s, invent[i].retName()))
        key = i;
    }
  }

  return key; // if key = -1, then string was not found
}

void Myr :: stats()
{
  cout << "--------------------------------" << endl
       << "| Myrlandira's Character Stats |" << endl
       << "--------------------------------" << endl
       << "  Life Points : " << life << endl
       << "  Essence : " << essence << endl
       << "--------------------------------" << endl
       << "| Spells                       |" << endl
       << "--------------------------------" << endl;
       if (numspells == 0)
         cout << "You know no spells." << endl;
       else
       {
         for (int i = 0; i < numspells; i++)
           cout << " " << spell_array[i].retName() << endl;
       }

  cout << "--------------------------------" << endl << endl;
}


// prints error message that spell cannot be cast
void Myr :: nope(char * spell)
{
  cout << "You don't have enough Essence to cast " << spell << endl;
}

void Myr :: incVar(int & variable)
{
  variable++;
}

void Myr :: decVar(int & variable)
{
  variable--;
}

void Myr :: setVar(int & variable, int num)
{
  variable = num;
}



// ----------------------------------------------------------------
// SPELLS METHODS
// ----------------------------------------------------------------

void Myr :: addSpell(char * spellname, int spellcost)
{
  spell_array[numspells].newSpell(spellname, spellcost);
  incVar(numspells);
}

void Myr :: castSpell(char * s, int & casted)
{
  int key = -1;

  key = findSpell(s);

  if (key == -1)
    cout << "You don't know that spell." << endl;
  else
  {
    if (essence > spell_array[key].retSC())
    {
      casted = 1;
      essence = essence - spell_array[key].retSC();
      spell_array[key].setActivated(1); // activate the spell
    }
    else
      nope(s);
  }
}

int Myr :: findSpell(char * s)
{
  int key = -1;

  for (int i = 0; (i < numspells); i++)
  {
    if(!strcmp(s, spell_array[i].retName()))
      key = i;
  }

  return key; // if key = -1, then string was not found
}

void Myr :: setNumspells(int num)
{
  numspells = num;
}

int Myr :: retNumspells()
{
  return (numspells);
}

char * Myr :: retSpellname(int key)
{
  return (spell_array[key].retName());
}

int Myr :: retSpellcost(int key)
{
  return (spell_array[key].retSC());
}


int Myr :: retSpellState(char * s)
{
  int key = -1;

  key = findSpell(s);
  if (key == -1)
    cout << "" << endl;

  return (spell_array[key].retActivated());
}


// ----------------------------------------------------------------
// COMBAT METHOD FUNCTIONS
// ----------------------------------------------------------------
int Myr :: retCS()
{
  return cs;
}

int Myr :: retWS()
{
  return ws;
}

int Myr :: retDefense()
{
  return defense;
}

void Myr :: setBonus(int num)
{
  bonus = num;
}
  
int Myr :: retBonus()
{
  return bonus;
}
