 /*******************************
 * general.h
 * File created: 23. June 1999
 * Last modified: 10. August 1999
 *******************************
 * General set of functions
 * which will be available in
 * most rooms.
 */

// include libraries
#include <fstream.h>

/*********************************************************
 * initxyz(char string[])                                *
 * initialize the tokens to clear them out               *
 ********************************************************/
void initxyz(char string[])
{
  for (int i = 0; i < 10; i++)
    string[i] = '\0';
}

/*********************************************************
 * init_array(char string[], int size)                   *
 * initialize the array                                  *
 ********************************************************/
void init_array(char string[], int size)
{
  for (int i = 0; i < size; i++)
    string[i] = '\0';
}

/*********************************************************
 * ginventory(Myr wytch)                                 *
 * Gives the inventory of the wytch                      *
 ********************************************************/
void ginventory(Myr wytch)
{
  wytch.printInv();
}

/*********************************************************
 * gstats(Myr wytch)                                     *
 * Gives the stats of the character                      *
 ********************************************************/
void gstats(Myr wytch)
{
  wytch.stats();
}

/*********************************************************
 * savegame()                                            *
 * Saves game and character stats                        *
 ********************************************************/
void savegame(int setroom, Myr wytch, Room rooms[])
{
  char filename[20] = "saved";
  char longfilename[24];

  cout << "Name your saved game: ";
  cin >> filename;

  strcpy(longfilename, filename);
  strcat(longfilename, ".sav");

  ofstream output(longfilename, ios:: out);
  ofstream savefile("saved.dat", ios:: app);

  if (!output) // exit on error
  {
    cout << "Unknown error." << endl;
    return;
  }
  else
  {
    output << setroom << endl; // current room number
    output << wytch.retLife() << endl; // wytchs LP
    output << wytch.retEssence() << endl; // wytchs EP
    output << wytch.retCircle() << endl; // wytchs Circle
    output << wytch.retNumspells() << endl; // # of spells known
    for (int i = 0; i < wytch.retNumspells(); i++) // saving spells
    {
      output << wytch.retSpellname(i) << endl;
      output << wytch.retSpellcost(i) << endl;
    }   
    // also need to add wytch inventory, etc.
    savefile << filename << endl;
  }
  // file is closed automatically 
}

/*********************************************************
 * restoregame()                                         *
 * Restores the game and character stats                 *
 *********************************************************
 * Notes on file input/output:                           *
 * use >> if reading in a number or string w/ no spaces  *
 * use getline if reading in a string w/ spaces          *
 * be sure to clear out the preceeding CR and newline ch *
 ********************************************************/
void restoregame(int & setroom, Myr & wytch, Room rooms[], char breakloop[])
{
  char filename[20];
  char longfilename[24];
  int buffer1;  
  int lp = 0; // life points
  int ep = 0; // essence points
  int circle = 0; // Circle of Magic
  int numberspells = 0;
  char spellname[20];
  int spellcost = 0;
  ifstream savefile("saved.dat", ios:: in); // open saved.dat

  init_array(spellname, 20); // initialize array spellname
  init_array(longfilename, 24);

  cout << "Saved Games" <<
  endl << "-----------" << endl;
  while (savefile >> filename) // while not at end of file
  {
    cout << filename << endl;
  }
  cout << "-----------" << endl;


  cout << "Enter in the name of your saved game: ";
  cin >> filename;
  strcpy(longfilename, filename);
  strcat(longfilename, ".sav");

  ifstream input(longfilename, ios:: in);

  if(!input)
    cout << "File does not exist." << endl;
  else
  {
    wytch.setNumspells(0); // initialize numspells to 0
    input >> setroom;
    input >> lp;
    input >> ep;
    input >> circle;
    input >> numberspells;
    if (numberspells > 0)
    {
      for (int i = 0; i < numberspells; i++)
      {
        input >> spellname;
        input >> spellcost;
        wytch.addSpell(spellname, spellcost);
        init_array(spellname, 20); // initialize spellname
        spellcost = 0; // initialize spellcost
      }
    } 
    wytch.setLife(lp);
    wytch.setEssence(ep);
    wytch.setCircle(circle);
  }


  strcpy(breakloop, "bye");
}

/*********************************************************
 * help()                                                *
 * Help function to aid the user in playing the game     *
 ********************************************************/
void help()
{
  cout <<
  endl << "HELP" <<
  endl << "----" <<
  endl << "Here is a list of many words and abbreviations the game " <<
  endl << "will understand: " << endl << 
  endl << "cast close d down drop e east floor get ground look n ne"<<
  endl << "north northeast open quit read restore s save search sign"<<
  endl << "south talk take w west u up" << endl <<
  endl << "For further help, type 'tutorial' for game instructions."<<
  endl << endl;
}

/****************************************************************
* scanprep(char [][7], char [], char [], char [], char [], int) *
* Written: 4. July 1999					   	*
* Scans the command line and removes any unnecessary words 	*
* such as prepositions and articles			   	*
****************************************************************/
void scanprep(char token[], char token2[], char token3[], char token4[])
{
  char prep[20][7] = {"a", "at", "of", "the", "to", "with"};
  int numprep = 6; // number of prepositions & articles
  int i = 0;

  for (i = 0; i < numprep; i++)
  {
    if (!strcmp(token, prep[i]))
    {
      strcpy(token, token2);
      strcpy(token2, token3);
      initxyz(token4);
    }    
  }

  if (strcmp(token2, "\0"))
  {
    for (i = 0; i < numprep; i++)
    {
      if (!strcmp(token2, prep[i]))
      {
        strcpy(token2, token3);
        strcpy(token3, token4);
        initxyz(token4);
      }
    }

    if (strcmp(token3, "\0"))
    {
      for (i = 0; i < numprep; i++)
      {
        if (!strcmp(token3, prep[i]))
        {
          strcpy(token3, token4);
          initxyz(token4);
        }
      }
    }
  }
}

/****************************************************************
* drink(Myr & wytch, char token2[], char token3[], int num)     *
* Written: 1. August 1999					*
* Last modified: 5. August 1999                                 *
* General function in drinking potions. 'num' is for the number *
* of EP or LP the potion will affect, whether good or bad       *
****************************************************************/
void drink(Myr & wytch, char token2[], char token3[], int num)
{
  if (!strcmp(token2, "green") && !strcmp(token3, "potion"))
  {
    if (wytch.searchInv("green potion") != 0)
    {
      cout << "You choke as you force the bile, green liquid down your"<<
      endl << "throat.  You quickly keel over and die from the poison."<<
      endl << "You are dead...thank you for playing." << endl;
      wytch.removeInv("green potion");
      exit(1);
    }
  }
  else if (!strcmp(token2, "elixer"))
  {
    if (wytch.searchInv("elixer") != 0)
    {
      cout << "You feel great relief after drinking the elixer." << endl;
      wytch.modLife(8);
      wytch.removeInv("elixer");
    }
    else
      cout << "You don't have an elixer potion." << endl;
  }
  else if ((!strcmp(token2, "healing") || !strcmp(token2, "blue")) && !strcmp(token3, "potion"))
  {
    if (wytch.searchInv("healing potion") != 0 || wytch.searchInv("blue potion") != 0)
    {
      cout << "You feel the healing properties of the potion coursing through"<<
      endl << "your body as you drink the potion." << endl;
      wytch.modLife(4);
      if (!strcmp(token2, "healing"))
	wytch.removeInv("healing potion");
      else
	wytch.removeInv("blue potion");
    }
    else
      cout << "You don't have a " << token2 << " potion." << endl;
  }
  else if ((!strcmp(token2, "essence") || !strcmp(token2, "red")) && !strcmp(token3, "potion"))
  {
    if (wytch.searchInv("essence potion") != 0 || wytch.searchInv("red potion") != 0)
    {
      cout << "You sense your Essence regaining after drinking that potion." <<
      endl;
      wytch.modEssence(3);
      if (!strcmp(token2, "essence"))
	wytch.removeInv("essence potion");
      else
	wytch.removeInv("red potion");
    }
    else
      cout << "You don't have a " << token2 << " potion." << endl;
  }
  else if (!strcmp(token2, "clear") && !strcmp(token3, "potion"))
  {
    cout << "Wow, it's a blast from the past!  This clear liquid is a true"<<
    endl << "relic...it tastes not unlike Crystal Pepsi." << endl;
    wytch.removeInv("clear potion");
  }
  else
    cout << "";
}


/****************************************************************
* attack(Myr & wytch, Enemy & monster)                          *
* Written: 3. August 1999                                       *
* Last modified: 11. August 1999			       	*
****************************************************************/
void attack(Myr & wytch, Enemy & monster)
{
  int total_damage = 0;
  int init_damage = 0;
  int combat_result = 0;
  int ot = wytch.retCS() + wytch.retWS() + wytch.retBonus();
  int dt = monster.retDSkill();
  int randomnum = get_random(0, 14);
  int tohit = get_random(0, 14);
  int table[11][15] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
		       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3,
		       0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 4,
		       0, 0, 0, 0, 0, 0, 0, 1, 1, 2, 5,
		       0, 0, 0, 0, 0, 1, 1, 2, 2, 3, 6,
		       0, 0, 0, 0, 1, 2, 2, 3, 3, 4, 7,
		       0, 0, 1, 1, 2, 3, 3, 4, 4, 5, 8,
		       0, 1, 2, 2, 3, 4, 4, 5, 5, 6, 9,
		       0, 2, 3, 3, 4, 5, 5, 6, 6, 7, 10,
		       1, 3, 4, 4, 5, 6, 6, 7, 7, 8, 12,
		       2, 4, 5, 5, 6, 7, 7, 8, 8, 9, 14,
		       3, 5, 6, 6, 7, 8, 8, 9, 9, 10, 15,
		       4, 6, 7, 7, 8, 9, 9, 10, 10, 11, 100,
		       8, 9, 10, 11, 12, 13, 14, 15, 16, 100, 100};

  combat_result = ot - dt;
  combat_result = combat_result / 2 + 5; // modify for battle table

  if (combat_result > 10)
    combat_result = 10;

  if (combat_result < 0)
    combat_result = 0;

  /* If the random number tohit is lower than or equal to the combat_result
     then the hit was successful */
  if (tohit <= combat_result)
  {
    total_damage = table[combat_result][randomnum] * -1;

    cout << "You injure the " << monster.retName() << " for " << 
            table[combat_result][randomnum] << " points of damage." << endl;
    monster.setLP(total_damage);
  }
  else
    cout << "You swing and miss your opponent." << endl;

}


/****************************************************************
* spellattack(Myr & wytch, Enemy & monster)                     *
* Written: 3. August 1999                                       *
* Last modified: 11. August 1999		       		*
****************************************************************/
void spellattack(Myr & wytch, Enemy & monster)
{
  int casted = 0; // initially the spell has not been cast yet
  int damage = 0;

  cout << "-----------------------------" <<
  endl << "| Spells                    |" <<
  endl << "-----------------------------" << endl;
  if (wytch.retNumspells() == 0)
    cout << "You don't know any spells." << endl;
  else
  {
    for (int i = 0; i < wytch.retNumspells(); i++)
      cout << " " << wytch.retSpellname(i) << endl;
  }

  char command[25], token[10], token2[10], token3[10];
  char breakloop[4] = "";

  printf("\nCast which spell > ");

  /* initialize the tokens */
  initxyz(token);
  initxyz(token2);
  initxyz(token3);

  gets(command);
  sscanf(command, "%s %s %s", token, token2, token3);


  if (!strcmp(token, "cast"))
  {
    strcpy(token, token2);
    strcpy(token2, token3);
    strcpy(token3, "\0");
  }

  // concatenate token & token 2
  if (strcmp(token2, "\0")) // if token3 is not NULL, then...
  {
    strcat(token, " ");
    strcat(token, token2);
  }

  wytch.castSpell(token, casted);

  if (casted == 1)
  {
    if (!strcmp(token, "Aura"))
    {
      cout << "You cast the Aura spell." << endl;
      wytch.setBonus(2); // causes 2 more damage points
    }
    else if (!strcmp(token, "Acid Burn"))
    {
      damage = -1 * get_random(3, 7);
      cout << "Your spell burns your enemy for " << damage << " points" <<
      endl << "of damage." << endl;
      monster.setLP(damage);
    }
    else if (!strcmp(token, "Fire Spear"))
    {
      cout << "You lance your Fire Spear out and hit for 7 " <<
      endl << "points of damage." << endl;
      monster.setLP(-7);
    }
    else if (!strcmp(token, "Fire Wind"))
    {
      cout << "A great sheet of fire-laced wind surrounds the" <<
      endl << "the " << monster.retName() << " and strikes for 25" <<
      endl << "points of damage." << endl;
      damage = -1 * get_random(10, 25);
      monster.setLP(damage);
      if (wytch.retLife() <= 5)
      {
	cout << "The strain from casting that last spell, in addition" <<
	endl << "to your other wounds has proven to be too much and you"<<
	endl << "die from exhaustion." << endl;
      }
      else
	wytch.modLife(-5);
    }
    else if (!strcmp(token, "Psychic Blast"))
    {
      cout << "Your spell strikes your enemy for 2 points of damage." <<
      endl;
      monster.setLP(-2);
    }
    else if (!strcmp(token, "Psychic Block"))
    {
      cout << "You ward yourself against psychic assault with your spell."<<
      endl;
    }
    else if (!strcmp(token, "Drain"))
    {
      damage = -1 * get_random(2, 8);
      cout << "You drain " << damage << " points of Life from your enemy." <<
      endl;
      monster.setLP(damage);
    }
    else
      cout << "That spell wasn't a battle spell." << endl;
  }
}

/****************************************************************
* attack2(Enemy & monster, Myr & wytch)                         *
* Written: 3. August 1999                                       *
* Last modified: 11. August 1999				*
* Enemy's attack, specifically for the Psychomancer             *
****************************************************************/
void attack2(Enemy & monster, Myr & wytch)
{
  int total_damage = 0;
  int init_damage = 0;
  int combat_result = 0;
  int ot = monster.retCSkill() + monster.retWSkill() + monster.retBPoints();
  int dt = wytch.retDefense();
  int randomnum = get_random(0, 14);
  int tohit = get_random(0, 14);
  int whichattack = get_random(1, 6); // determine which attack
  int table[11][15] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
		       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3,
		       0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 4,
		       0, 0, 0, 0, 0, 0, 0, 1, 1, 2, 5,
		       0, 0, 0, 0, 0, 1, 1, 2, 2, 3, 6,
		       0, 0, 0, 0, 1, 2, 2, 3, 3, 4, 7,
		       0, 0, 1, 1, 2, 3, 3, 4, 4, 5, 8,
		       0, 1, 2, 2, 3, 4, 4, 5, 5, 6, 9,
		       0, 2, 3, 3, 4, 5, 5, 6, 6, 7, 10,
		       1, 3, 4, 4, 5, 6, 6, 7, 7, 8, 12,
		       2, 4, 5, 5, 6, 7, 7, 8, 8, 9, 14,
		       3, 5, 6, 6, 7, 8, 8, 9, 9, 10, 15,
		       4, 6, 7, 7, 8, 9, 9, 10, 10, 11, 100,
		       8, 9, 10, 11, 12, 13, 14, 15, 16, 100, 100};

  if (whichattack <= 2)
  {
    combat_result = ot - dt;
    combat_result = combat_result / 2 + 5;

    if (combat_result > 10)
      combat_result = 10;

    if (combat_result < 0)
      combat_result = 0;
  
    /* If the random number tohit is lower than or equal to the combat_result
       then the hit was successful */
    if (tohit <= combat_result)
    {
      total_damage = table[combat_result][randomnum] * -1;
      wytch.modLife(total_damage);

      cout <<
      endl << "You are injured for " << table[combat_result][randomnum] <<
              " points of damage." << endl;
    }
    else
    {
      cout <<
      endl << "The Psychomancer swings his dagger at you and misses." <<endl;
    }
  }
  else if (whichattack <= 5)
  {
    init_damage = get_random(5, 10);
    cout <<
    endl << "The Psychomancer spears a Psychic Blast towards you." << endl;
    if (wytch.retSpellState("Psychic Block") == 1)
    {
      cout << "Your Psychic Block spell deflects some of the damage and"<<
      endl << "you are hit for only " << init_damage/2 << " points of "<<
              "damage." << endl;
      total_damage = init_damage/2 * -1;
      wytch.modLife(total_damage);
      monster.setEP(-6);
    }
    else
    {
      cout << "You are struck for " << init_damage << " points of " <<
              "damage." << endl;
      total_damage = init_damage * -1;
      wytch.modLife(total_damage);
      monster.setEP(-6);
    }
  }
  else
  {
    init_damage = get_random(3, 15);
    total_damage = init_damage * -1;
    wytch.modLife(total_damage);
    monster.setEP(-7);
    cout <<
    endl << "The Psychomancer casts a Surge spell at you, injuring you " <<
    endl << "for " << init_damage << " points of damage." << endl;
  }


}

/****************************************************************
* battle(Myr & wytch, Enemy & monster)                          *
* Written: 3. August 1999                                       *
* Last modified: 5. August 1999					*
****************************************************************/
void battle(Myr & wytch, Enemy & monster)
{
  int choice = 0;
  int firstround = 1;


  while (monster.retLP() > 0)
  {
    cout <<
    endl << "------------------------------------------------------" <<
    endl << " Myrlandria            | " << monster.retName() <<
    endl << "------------------------------------------------------" <<
    endl << " LP : " << wytch.retLife() << "                  LP : " << monster.retLP() << 
    endl << " EP : " << wytch.retEssence() <<"                  EP : " << monster.retEP() <<
    endl << "------------------------------------------------------" <<
    endl << 
    endl << "1. Attack " <<
    endl << "2. Cast a spell" <<
    endl << "> ";

    get_int(choice, 1, 2);

    switch(choice)
    {
    case 1 : attack(wytch, monster); break;
    case 2: spellattack(wytch, monster); break;
    default: cout << "That wasn't a choice." << endl;
    }

    if (monster.retLP() > 0)
    {
      if (firstround == 1)
      {
	int init_damage = get_random(5, 10);
        int final_damage = 0;
	cout <<
	endl << "The Psychomancer spears a Psychic Blast towards you." << endl;
	if (wytch.retSpellState("Psychic Block") == 1)
	{
	  cout << "Your Psychic Block spell deflects some of the damage and"<<
	  endl << "you are hit for only " << init_damage/2 << " points of "<<
	          "damage." << endl;
	  final_damage = init_damage/2 * -1;
	  wytch.modLife(final_damage);
	  firstround = 0; // no longer first round
	}
	else
	{
	  cout << "You are struck for " << init_damage << " points of " <<
	          "damage." << endl;
	  final_damage = init_damage * -1;
	  wytch.modLife(final_damage);
	  firstround = 0;
	}
      }
      else
        attack2(monster, wytch); // enemys turn to attack
    }
    else
    {
      cout << "The " << monster.retName() << " cries out and dies as you" << 
      endl << "make your final strike." << endl;
    }
  }
}
